/*
 * nsjc.java
 *
 * Created on October 25, 2002, 12:06 PM
 */

import java.io.*;
import java.net.*;
import java.lang.*;

/** Name: nsjc - Netsaint Java Client
 *
 *    Desc:
 *  This is the companion piece that goes with nsjsd (Netsaint Java Server).
 *  The nsjc is the client that connects to the nsjsd and will report to the service
 *  the status of a test. It is yet another application that is meant to enhance the
 *  passive service reporting capability of the Nagios application. I wrote these
 *  objects because I was frustrated with not being able to report status of my NT
 *  boxes to my host server which was FreeBSD. I don't want to criticise the efforts
 *  of those developers who spent time developing those applications which plug into
 *  Nagios and help server information back to the server, it's just that for some
 *  reason, I couldn't get them to work. Not being an expert in BSD and C programming
 *  I decided to write something myself that I could control and understand.
 *
 *  Besides the frustration of trying to get the NT reporting software to work with my
 *  systems, I do have some unique requirements. Not only do I have a mixed environment
 *  of NT systems but I also have to worry about Novell server's as well. I haven't seen
 *  anything out there that works with Novell, with the exception of the Perl scripts.
 *  I could have used the perl scripts for reporting the Passive Service information but
 *  again, I decided to work with something that I knew (and besides I'm finishing up
 *  my Data Structures and Algorithms class in Java and took this as a challenge). So
 *  I've written these objects with the intent that not only could they be used as
 *  standalone pieces but they can also be reused in other Java applications and
 *  applets as is.
 *
 *    Goal:
 *  The goal of this project is to provide a set of reusable objects that can either be
 *  incorporated into another application or can be used standalone via the command line.
 *
 *    Component Strategy:
 *  The nsjsd listens on a set of ports and will only accept connections from specific
 *  ip addresses and on specific ports (see the nsjsd code header). The nsjc will
 *  attempt to connect to the server running the nsjsd on a specific port. Once it connects
 *  it will attempt to send status information on a specific test. This status information
 *  will then be sent to the Nagios command file, for Nagios to process. This means
 *  your test information must be configured properly in the Nagios configuration files
 *  (see the Nagios documentation for details).
 *
 *    Object Architecture:
 *  This object holds several internal variables. These variables include host, test name,
 *  test results, test comments, port, and thread timeout values. The host indicates which
 *  server to connect to, the port is which port to connect on. The test information is
 *  the information that is sent to the server and placed in the Nagios command file.
 *  The thread timeout value is the time in milliseconds that the code will wait for
 *  the object to complete it's communication with the server. You do not have to set
 *  the thread timeout value, as it defaults to 5000 milliseconds. The port defaults
 *  to 27000.
 *
 *  If run from the command line the object will first attempt to parse the command line
 *  arguments. Note that the port and thread timeout values are optional. Once the command
 *  line arguments are interpreted it will instantiate a new nsjc object and then call the
 *  send method. The send method launches a new thread then waits for that thread to
 *  complete. It will only wait the time set by the timeout value, so if you have a slow
 *  network and the component is timing out, you may want to up this value. However
 *  5 seconds is probably more than enough.
 *
 *  If run inside another application then you'll want to instantiate the nsjc object
 *  using one of the constructor methods, there are three. One constructor allows
 *  you to set the host, port, timeout, and test data upon instantiation. The other
 *  two constructors allow you build the nsjc object without port or without the timeout
 *  value. Once the object has been instantiate then call the send method. The send
 *  method will return a boolean which will notify the caller of it's success.
 *
 *    Notes:
 *  Currently you have to instantiate the object with all the necessary data. The
 *  instance variables are all private and can't be changed after object instantiation.
 *  (At least in this version.) There is also a setting that will allow the programmer
 *  to stop command line output. By default if this object is run from the command line
 *  it will print output, however you can instantiate the object with command line output
 *  off.
 *
 *    Comments:
 *  Please keep in mind that these are very basic components
 *  and that this is my first attempt at writing a client/server applet in Java.
 *
 *    License
 * The is application is freely distributed. I only ask that if you incorporate or reuse
 * any of these components you make mention of where you got them from and my email
 * address.
 * 
 * Nagios is a registered trademark of Ethan Galstad.
 *
 * @author viaggi@nietzsche.net
 * @version 1.0.0
 * @ide NetBeans IDE 3.4
 */

public class nsjc implements Runnable{
    
    /** Port used if no other port is passed into the object */    
    public static final int DEFAULT_PORT = 27000;
    /** Default thread timeout in milliseconds. This is the time the code will wait for
     * the thread to complete it's task before it kills it.
     */    
    public static final long THREAD_TIMEOUT_DEFAULT = 5000;
    
    private String ivar_host = new String();
    private String ivar_test = new String();
    private String ivar_data = new String();
    
    private int ivar_port = DEFAULT_PORT;
    private int ivar_status = -1;
    private long ivar_thread_timeout = THREAD_TIMEOUT_DEFAULT;
    private boolean ivar_sent = false;
    private boolean ivar_print_status = true;
    
    /** Name: run
     * Desc: this is the manditory run method, as this object implements Runnable.
     * This method is launched when the thread is started after the object is instantiated.
     * It's only function is to call an internal method which will attempt to send the 
     * data to the host.
     */
    public void run(){
        this.sendData();
    }
    
    /** Name: usage
     * Desc: this method is called when the 'main' method has problems interpreting the 
     * command line arguments or perceives an error in them. It will write out to the 
     * command line usage information.
     */
    public static void usage(){
        System.out.println("");
        System.out.println("");
        System.out.println("I'm guessing that you need a little help with the command line arguments:");
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("Usage: java Client <hostname> <desc> <return_code> <plugin_output> <port> <timeout>");
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("");
        System.out.println("");
        System.out.println("   <hostname>      - the name of the server to send the report to.");
        System.out.println("   <desc>          - description is the description of the service as ");
        System.out.println("                     specified in the service definition .");
        System.out.println("   <return_code>   - the return code of the check (0=OK, 1=WARNING, ");
        System.out.println("                     2=CRITICAL, 3=UNKNOWN).");
        System.out.println("   <plugin_output> - the text output of the service check (i.e. the ");
        System.out.println("                     plugin output.");
        System.out.println("   <port>          - OPTIONAL - the port the server is listening on");
        System.out.println("                     defaults to: 27000.");
        System.out.println("   <timeout>       - OPTIONAL - the thread timeout in milliseconds.");
        System.out.println("                     defaults to: 5000.");
        System.out.println("");
        System.out.println("");
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("author: viaggi@nietzsche.net");
        System.out.println("version: 1.0");
        System.out.println("date: December 2002");
        System.out.println("Note: more docuementation can be extracted using javadoc.");
        System.out.println("");
        System.exit(0);
    }
    
    /* Name: sendData
     * Desc: This method is the meat of the object. It will atempt to connect to the server
     * on the port specified and send the server the information regarding the test. It will
     * then set an internal instance variable to either true or false, depending upon 
     * whether or not it is successful in delivering the information. It also returns
     * as a boolean it's success status as well.
     * Pre: true
     * Post: true
     * 
     * @return boolean -> true, if it successfully connects and sends it's information to the server.
     */
    private boolean sendData(){
    
        Socket sktNSJC = null;                          //Socket to communicate on
        
        String strSend = new String();                  //Holder for the String info to send
        StringBuffer stbSend = new StringBuffer();      //Holder to construct the send String
        
       try
        {
            //Send String: test;
            stbSend.append(ivar_test);
            stbSend.append(";");
            
            //Connect to the Host on the Port
            sktNSJC = new Socket(ivar_host, ivar_port);
            
            //Open up a Data Input Stream
            DataInputStream disNSJC = new DataInputStream(sktNSJC.getInputStream());
            //Open up an Print Stream
            PrintStream psNSJC = new PrintStream(sktNSJC.getOutputStream());

            //The following two lines are for bi-directional communication
            //DataInputStream disIn = new DataInputStream(System.in);
            //BufferedReader brOut  = new BufferedReader(new InputStreamReader(disIn));

            //Open a Buffer Reader
            BufferedReader brIn  = new BufferedReader(new InputStreamReader(disNSJC));
            
            //Print confirmation of connection
            if(ivar_print_status == true)
            {System.out.println("Connected to " + sktNSJC.getInetAddress() + ":" + sktNSJC.getPort());}
            
            //Send the data to the server as three separate items
            psNSJC.println(ivar_test);
            psNSJC.println(ivar_status);
            psNSJC.println(ivar_data);
            
            //Read the Buffered Reader to see if everything's been sent
            String line;
            line = brIn.readLine();
            if(line==null)
                {ivar_sent = false;}
            else
                {ivar_sent = true;}

            return ivar_sent;
        }
        catch (IOException e)
        {
            System.err.println(e);
            ivar_sent = false;
            return false;
        }
        finally
        {
            //Close the socket
            try 
            {if (sktNSJC != null) sktNSJC.close();}
            catch(IOException e2)
            {return false;}
        }
    }
    
    /** Name: main
     * Desc: this is the main routine that will run when this object is instantiated via
     * command line. It will attempt to parse the arguments that are passed in and then
     * it will insantiate a new nsjc object and then call it will call the send method
     * (which attempts the communication with the server). If the object is run without
     * command line arguments it will print out usage information.
     * Pre: at least 4 valid command line arguments.
     * Post: true
     * 
     * @param args -> an array of strings which hold the command line arguments. It must
     * have at least four arguments for this routine to run. (Of course the arugments
     * must be valid too.)
     */
    public static void main(String[] args){
        
        int intResult = -1;
        int intPort = DEFAULT_PORT;
        long lngThreadTimeout = THREAD_TIMEOUT_DEFAULT;
        boolean boolPrintStatus = true;
        
        String strHost = new String();
        String strTest = new String();
        String strData = new String();
        
        //Check the number of command line arguments
        if (args.length < 4) 
        {usage();}

        //Validate the 3 argument in the array, is it a valid result?
        try
        {
                intResult = Integer.parseInt(args[2]);
                if(intResult < 0)
                {usage();}
                if(intResult > 3)
                {usage();}
        }
        catch (NumberFormatException e)
            {usage();}
        
        //Extract the host, test, and test data from the array
        strHost = args[0];
        strTest = args[1];
        strData = args[3];
        
        //Check to see if there is a 5th command line argument 
        //If there is extract the port
        if (args.length == 5)
        {
            try
                {intPort = Integer.parseInt(args[4]);}
            catch (NumberFormatException e)
                {usage();}
        }

        //Check to see if there is a 6th command line argument
        //If there is extract the port and the thread timeout value
        if (args.length == 6)
        {
            try
                {intPort = Integer.parseInt(args[4]);}
            catch (NumberFormatException e)
                {usage();}

            try
                {lngThreadTimeout = Long.parseLong(args[5]);}
            catch (NumberFormatException e)
                {usage();}
        }
        
        //Instantiate a new nsjc object and run it.
        nsjc nsRun = new nsjc(strHost, intPort, strTest, intResult, strData, lngThreadTimeout, boolPrintStatus);
        nsRun.send();
        
    }
    
    /** Name: send
     * Desc: This method will instantiate a new thread and then waits for the thread.
     * Remember that a instantiating a new thread and running it, starts the communication
     * process within the object. If the thread is still running when the wait timeout
     * expires the method will destroy the thread and return false. After this is done it 
     * will print to the command line it's status. It will also return it's status.
     * 
     * @return boolean -> true if the object was successful in exectuing it's code. False
     * if it had to kill the running thread.
     */
    public boolean send(){
        
        try
        {
            //Initiate the thread
            Thread thrRun = new Thread(this);
            //Start the thread
            thrRun.start();
            //Wait for the thread to finish
            thrRun.join(ivar_thread_timeout);
            
            //If the thread is still running after the wait timeout, then kill it
            if(thrRun.isAlive() == true)
            {
                //thrRun.interrupt();
                thrRun.destroy();
                
                if(ivar_print_status == true)
                {System.out.println("Thread Timeout activated.");}

                ivar_sent = false;
            }
        }
        catch(InterruptedException ie)
        {
            if(ivar_print_status == true)
            {System.out.println("Send Thread Interrupted");}
        }
        finally
        {
            //Print success to the command line.
            if(ivar_sent == true)
            {
                if(ivar_print_status == true)
                {System.out.println("Delivered successfully.");}
            }
            else
            {
                if(ivar_print_status == true)
                {System.out.println("Delivery unsuccessfull.");}
            }
            
            return ivar_sent;
        }
        
    }
    
    /** Name: nsjc
     * Desc: Constructor to instantiate and initialize the object. Using this method will
     * make the object use the default values for port and for thread timeout. 
     * 
     * @param strHost -> String, the host name you want to connect to (could be an ip address)
     * @param strTest -> String, the name of the test (should match the test name in the Nagios config file)
     * @param intStatus -> Must be either -1 UNKNOWN, 0 OK, 1 WARNING, or 2 CRITICAL
     * @param strData -> String, extra data to send regarding the test
     * @param boolPrintStatus -> boolean, true will print status information to the command line.
     */
    public nsjc(String strHost, String strTest, int intStatus, String strData, boolean boolPrintStatus){
    
        ivar_host = strHost;
        ivar_test = strTest;
        ivar_data = strData;
        ivar_status = intStatus;
        ivar_print_status = boolPrintStatus;
        
    }

    /** Name: nsjc
     * Desc: Constructor to instantiate and initialize the object. Using this method will
     * make the object use the default values for thread timeout. 
     * 
     * @param strHost -> String, the host name you want to connect to (could be an ip address)
     * @param intPort -> int, the port to connect to on the host
     * @param strTest -> String, the name of the test (should match the test name in the Nagios config file)
     * @param intStatus -> Must be either -1 UNKNOWN, 0 OK, 1 WARNING, or 2 CRITICAL
     * @param strData -> String, extra data to send regarding the test
     * @param boolPrintStatus -> boolean, true will print status information to the command line.
     */
    public nsjc(String strHost, int intPort, String strTest, int intStatus, String strData, boolean boolPrintStatus){
    
        ivar_host = strHost;
        ivar_test = strTest;
        ivar_data = strData;
        ivar_port = intPort;
        ivar_status = intStatus;
        ivar_print_status = boolPrintStatus;

    }

    /** Name: nsjc
     * Desc: Constructor to instantiate and initialize the object. 
     * 
     * @param strHost -> String, the host name you want to connect to (could be an ip address)
     * @param intPort -> int, the port to connect to on the host
     * @param strTest -> String, the name of the test (should match the test name in the Nagios config file)
     * @param intStatus -> Must be either -1 UNKNOWN, 0 OK, 1 WARNING, or 2 CRITICAL
     * @param strData -> String, extra data to send regarding the test
     * @param lngThreadTimeout -> long, the time in milliseconds for the code to wait for the thread to complete
     * @param boolPrintStatus -> boolean, true will print status information to the command line.
     */
    public nsjc(String strHost, int intPort, String strTest, int intStatus, String strData, long lngThreadTimeout, boolean boolPrintStatus){
    
        ivar_host = strHost;
        ivar_test = strTest;
        ivar_data = strData;
        ivar_port = intPort;
        ivar_status = intStatus;
        ivar_thread_timeout = lngThreadTimeout;
        ivar_print_status = boolPrintStatus;
        
    }

}

