import java.net.*;
import java.io.*;
import java.util.*;

/** Name: Server
 * Desc: This object is designed to listen on a specific port for incomming 
 * connections from a nsjc client. One object is spawned per port by the nsjsd
 * application. 
 *
 * This object basically waits for a connection. When it receives one it will
 * attempt to identify the host. It does this by checking the alias records
 * and the host records in the system. If it doesn't find the host/alias in 
 * the list of records then it will refuse the connection. If it accepts the connection
 * then it will launch a Connection object to handle the transmission of the data.
 *
 * It is important to note that this object won't actually start listening for
 * connections until it is 'started'. Because this object extends Threads, call
 * the 'run' method. That will launch an asynchronise thread and start the Server object
 * listening on the port.
 *
 * @author viaggi@nietzsche.net
 */
public class Server extends Thread{
    
    private int ivar_port;
    private ServerSocket ivar_listen_socket;

    private nsjsd ivar_parent;
  
    /** Name: get Thread
     * Desc: This function will return the running thread as an object.
     * If it works during the while routine.
     *
     * @return Thread --> the current thread running this object
     */
    public Thread getThread(){
        
        Thread thrCurrent = new Thread();
        thrCurrent = Thread.currentThread();
        return thrCurrent;
    }
    
    /** Name: write log
     * Desc: Calls the nsjsd parent's write log method.
     * @param strMsg --> String, the message to write to the log file
     */
    public void writelog(String strMsg){
        ivar_parent.writelog(strMsg);
    }
   
    /** Name: fail
     * Desc: Calls the nsjsd parent's fail method.
     * @param exc --> Exception, the Exception to report on
     * @param strMsg --> String, the message to report
     */
    public void fail(Exception exc, String strMsg){
        ivar_parent.fail(exc, strMsg);
    }
  
    //Name: acceptable host
    //Desc: This method is used to match the client that just connected
    //      to the acceptable list of clients in either thos Hosts list or
    //      the Alias list.
    //Return: String --> returns the name of the host
      private String acceptablehost(Socket client_socket){

          int intHostCount;
          int i;
          int intPort;
          
          String strHostCanonical;
          String strHostIP;
          String strHostFull;
          
          String strAlias = new String();
          String strHost = new String();
          
          InetAddress iaddr;
          host hstTest;
          alias alTest;

          intPort = client_socket.getLocalPort();
          iaddr = client_socket.getInetAddress();
          strHostCanonical = iaddr.getCanonicalHostName();
          strHostFull = iaddr.getHostName();
          strHostIP = iaddr.getHostAddress();
          
          //Set the Host Default equal to the Host Full Name
          strHost = strHostFull;
          
          //First check all the aliases for a match on either the Canonical or Full Host name
          for(i=0;i < ivar_parent.getAlias().size();i++)
          {
              alTest = (alias)ivar_parent.getAlias().get(i);
              
              //If the canonical or full name matches an alias, then set the host string equal to the 
              //alias and break from the for loop.
              if((alTest.alias().equals(strHostCanonical)) || (alTest.hostname().equals(strHostFull)))
              {
                  strHost = alTest.hostname();
                  break;
              }
          }
          
          //if (strHost.indexOf(".") > 0)
          //{strHost = new String(strHost.substring(0, strHost.indexOf(".")));}

          //Loop through the authorized hosts to determine if the Host is valid
          //and can send information to the Server
          for(i=0;i < ivar_parent.getHosts().size();i++)
          {
              //Retrieve the host record
              hstTest = (host)ivar_parent.getHosts().get(i);
              
              //If the host record matches the port then proceed
              if (hstTest.port() == intPort) 
              {
                  //Validate that the host matches either the Host
                 if (hstTest.hostname().equals(strHost) == true)
                 {return strHost;}
              }
          }
          
          //If you've gotten this far, then obviously nothing matched,
          //return false.
          return new String();

      }

    /** Name: Server
     * Desc: The constructor of the Server object. This takes as parameters the 
     * port it should listen on and the nsjsd Parent which holds information regarding
     * the Hosts/Aliases and log files. 
     *
     * @param intPort --> int, the port to listen on
     * @param nsjParent --> najsd, the Parent which hold the configuration information
     */
      public Server(int intPort, nsjsd nsjParent){

          if (intPort == 0) 
          {intPort = nsjParent.DEFAULT_PORT;}

          ivar_parent = nsjParent;
          ivar_port = intPort;

          //Try opening a new socket.
          try 
            {ivar_listen_socket = new ServerSocket(intPort);}
          catch (IOException e) 
            {fail(e, "Exception creating server socket");}
      }
  
      /** Name: run
       * Desc: This function is an over-riding method for implementing Threads.
       * It will start the Socket listening and process any incomming messages.
       */
      public void run(){
          
        try
        {
            //Retrieving current active thread
            Thread thrCurrent = new Thread();          
            thrCurrent = Thread.currentThread();
          
            String strHostName = new String();
            
            //Write to the log that this Server is starting
            writelog("Server running on thread: " + thrCurrent.toString() + " listening on port " + ivar_port);
          
            //Endless loop, until server is shut down.
            while(true)
            {
                //Start accepting connections
                Socket client_socket = ivar_listen_socket.accept();
                
                strHostName = acceptablehost(client_socket);
                
                //Validate that the host is authorized to send information
                if (strHostName.length() > 2)
                {
                    //Write to the log this host was accepted and then launch
                    //a Connection object
                    Date dtNow = new Date();
                    this.writelog("----------Host resolved to " + strHostName + " connected from " + client_socket.getInetAddress().getCanonicalHostName() + "/" + client_socket.getInetAddress().getHostAddress() + ":" + client_socket.getPort() + " --> Accepted : " + dtNow.getTime());

                    //Create a new connection object and then start it.
                    Connection clientConnection = new Connection(client_socket, ivar_parent, strHostName);
                    clientConnection.start();
                }
                else
                {
                    //Close the socket and write to the log this host was rejected.
                    client_socket.close();
                    Date dtNow = new Date();
                    this.writelog("----------Host resolved to " + strHostName + " connected from " + client_socket.getInetAddress().getCanonicalHostName() + "/" + client_socket.getInetAddress().getHostAddress() + ":" + client_socket.getPort() + " --> Declined : " + dtNow.getTime());
                }
            }
        }
        catch (IOException excFail)
        {fail(excFail,"Exception while listening for connections");}
      }
  
}

