import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

/** Name: Nagios Java Server
 *
 *     Desc: 
 * This piece is desinged as a companion to Nagios (now called Nagios).
 * It is designed to help provide information to Nagios, send it information
 * regarding passive commands. This means that your Nagios installation should
 * be configured to accept passive service updates (see the Nagios documentation
 * regarding how to set that up).
 *
 *    Goal:
 *  The goal of this project is to provide a set of reusable objects that can either be
 *  incorporated into another application or can be used standalone via the command line.
 *
 * Much like the other passive service companion software already out there,
 * such as the perl piece, this system is two pieces. There is a server that listens
 * for connections and then there is a client that connects to the server and
 * sends information to it. This piece is the server, which means that it will
 * listen on certain designated ports for communications from authorized systems.
 * I say authorized because there is a configuration file which must be present
 * for this application to work. The configuration file must be passed in as a
 * command line arguement (it must be the complete path and file name).
 *
 * I opted to write this in java, as opposed to using one of the other applications,
 * because I was having problems getting the others to run, quite frankly.
 * My environment is FreeBSD (for the Nagios monitoring server), Novell and NT
 * servers as my monitored systems. This meant that, although most of my monitoring
 * would be done over NT, I'd need something for Novell. The perl app, would probably
 * have run but I figured if I wrote one in Java I'd have all my bases covered. I have
 * yet been able to get the NT monitors to work and have been frustrated with them.
 * Although, with the Java, I may not be able to provide as much information as I would
 * have with the NT monitors, I was able to get it working. (And it was a fun exercise.)
 *
 * This object has been written to be run from the command line and requires the
 * full path to the configuration file when instantiated. The client application
 * has been written such that it can be encorporated and reused in other applications.
 * It is a single object which has exposed methods allowing for manipulation
 * and communication with this server. For information on the client application
 * I suggest reading the java doc on the nsjc object.
 *
 * Currently this application can only be run from the command line, however I would
 * like to incorporate control features into the server in later versions. There
 * are four supporting objects which, together with the nsjsd class, make up this
 * application. All of the configuration information is loaded at startup and held
 * within the nsjsd object. This means that runtime changes could be made to the
 * configuration items, if it was written into the application.
 *
 * For use information you can just run the application without any command line
 * parameters. By doing this use information will print to the command line.
 *
 * This class uses four supporting classes to run: Connection, Server, alias, and host.
 * The host class holds information on which hosts can connect to this server and on which
 * port. The alias class holds informaiton on transforming the host information retrieved
 * from the connection, into a common name. The Server class is the class which does
 * the active listening. The Connection class is the class which accepts the connection
 * from the Server and interprets the information from the connection.
 *
 * The configuration file can contain four different types of items: command_file,
 * log_file, host, and alias. The file must contain the first two items, and at least
 * a single host. Aliases are optional. (For sample configuration file see the file
 * nsjsd.cfg-sample.) The command file is the file which the test results will be
 * written to (see the command file information in the Nagios documentation). The
 * log file will be the file that the nsjsd application will write information to.
 * The host item consists of the host name, a colon, followed by the port that the
 * host is allowed to connect on. The nsjsd application will launch one Server object
 * per port. So if you have 25 host records each with a different port, then you'll
 * spawn 25 Server objects. The alias record is there to help transform the connecting
 * machine's name into a name that is recognized by Nagios. This record type came
 * about because I have having problems with the Java object inetAddress, it wouldn't
 * return the name of the host in a very predictable manner. In some cases it would
 * return the host name fine, in other cases it would just return the IP address.
 * So you can now specify alias records. The alias record is the name of the alias, a colon,
 * followed by the actual host name (this actual host name, is the name used to match
 * to the Nagios host). So if you're system continually returns the IP address
 * as the host when you call the getCanonicalName from the inetAddress object you'll
 * be able to use an alias record to transform the IP address into a host name.
 *
 * If you make a change to the configuration file, you will need to stop and restart the
 * application as realtime updates aren't yet available and the file is read at startup.
 *
 *    License
 * The is application is freely distributed. I only ask that if you incorporate or reuse
 * any of these components you make mention of where you got them from and my email
 * address.
 * 
 * Nagios is a registered trademark of Ethan Galstad.
 *
 * @author viaggi@nietzsche.net
 * @version 1.0.0
 */
public class nsjsd{
    
    /** Default port that the server will monitor in the event none is specified
     */    
  public final static int DEFAULT_PORT=28000;
  private File fileConfig;
  private File fileCommand;
  private File fileLog;
  
  private Vector lstHosts = new Vector();
  private Vector lstAlias = new Vector();
  private Vector lstServers = new Vector();
  
  
  /** Name: get Command File
   * Desc: This method returns the command file obtained from the config file.
   * 
   * @return File --> the File object that represents the config file.
   */
  public File getCommandFile(){
       return fileCommand;
  }
  
  /** Name: get Hosts
   * Desc: This method is provided so that other objects can get the internal
   * Vector of Hosts at any time. This means that most configuration objects
   * can be changed (providing admin tools are written) and will take affect immediately.
   * 
   * @return Vector --> the list of Hosts from the configuration file
   */
    public Vector getHosts(){
        return lstHosts;
    }
    
  /** Name: get Alias
   * Desc: This method is provided so that other objects can get the internal
   * Vector of Aliases at any time. This means that most configuration objects
   * can be changed (providing admin tools are written) and will take affect immediately.
   * 
   * @return Vector --> the list of Alias info from the configuration file
   */
    public Vector getAlias(){
        return lstAlias;
    }

     /** Name: write Command
     * Desc: This function will write information to the command file as designated
     * in the config file. It is synchronized because the individual threads will
     * be calling this method to write to the file. This may prove to be a bottle
     * neck, if you have a lot of different threads running and accepting information
     * all the time. If this is the case, move this function to the Connection object. Just
     * note that is is meant to be a choking point, as I wanted to make sure that if 
     * there were a lot of writes to the file, they would be completed sequentially
     * and processes wouldn't write over other processes' information. This should
     * help maintain the integrity of the file. This method appends the date to the 
     * beginning of the string and a new line character to the end of the string
     * before writing it to the file.
     * 
     * @param strMsg --> String, the message to write to the command file
     */
    public synchronized void writeCommand(String strMsg){
        
        FileWriter fwCommand;
        
        Date dtNow = new Date();
        
        StringBuffer stbDate;
        String strLine = new String();
        
        try
        {
            //Open the Command File for writing
            fwCommand = new FileWriter(fileCommand, true);
            
            //Get the date and remove the milliseconds from the end
            stbDate = new StringBuffer("" + dtNow.getTime());
            stbDate.setLength(stbDate.length() - 3);

            //Append the Date within brackets and a space before the string
            strLine = new String("[" + stbDate.toString() + "] " + strMsg + "\n");
            
            //Append the Line to the End of the File
            fwCommand.write(strLine);
            
            //Close the Command file
            fwCommand.close();                   
                    
        }
        catch (IOException e)
        {writelog(e.toString());}
    }
                
  /** Name: fail
   * Desc: This function will print an error message to the command line along with
   * any exception information that is passed in with it. After printing the error
   * to the command line it will stop the code.
   * 
   * @param exc --> Exception, the exception that forced this function to be called
   * @param strMsg --> String, companion message to print to the command line.
   *
   */
  public synchronized static void fail(Exception exc, String strMsg){
    System.err.println(strMsg + ":" + exc);
    System.exit(1);
  }
  
  /** Name: print Command Line
   * Desc: This function will print to the command line usage information.
   */
  public static void printCommandLine(){
    System.out.println("");
    System.out.println("");
    System.out.println("I'm guessing that you need a little help with the command line arguments:");
    System.out.println("-------------------------------------------------------------------------");
    System.out.println("-------------------------------------------------------------------------");
    System.out.println("Usage: java <nsjsd.cfg>");
    System.out.println("-------------------------------------------------------------------------");
    System.out.println("");
    System.out.println("");
    System.out.println("   <nsjsd.cfg> - the complete path to the configuration file.");
    System.out.println("");
    System.out.println("");
    System.out.println("-------------------------------------------------------------------------");
    System.out.println("author: viaggi@nietzsche.net");
    System.out.println("version: 1.0");
    System.out.println("date: October 2002");
  }
  
    /** Name: write log
     * Desc: This function will write information to the log file as designated
     * in the config file. It is synchronized because the individual threads will
     * be calling this method to write to the log file. This may prove to be a bottle
     * neck, if you have a lot of different threads running and accepting information
     * all the time. If this is the case, move this function to the Server object. Just
     * note that is is meant to be a choking point, as I wanted to make sure that if 
     * there were a lot of writes to the file, they would be completed sequentially
     * and processes wouldn't write over other processes' information. This should
     * help maintain the integrity of the file. 
     * 
     * @param strMsg --> String, the message to write to the log file
     */
     public synchronized void writelog(String strMsg){
         
         FileWriter fwLog;
    
        try
        {
            fwLog = new FileWriter(fileLog, true);
            fwLog.write(strMsg);
            fwLog.write("\n");
            fwLog.close();
        }
        catch(IOException ioe)
        {}    
    }
   
    //Name: validate log file
    //Desc: This function checks to see if the log file already exists. If it doesn't then
    //      it will create a new one.
    private void validatelogfile(){
        
        FileWriter fwLog;
        try
        {
            if (fileLog.exists() == false)
            {fwLog = new FileWriter(fileLog);}
        }
        catch(IOException ioe)
        {}
    
    }
  
      //Name: extract Hosts
      //Desc: This function extracts the hosts and aliases from the 
      //      configuration file that was passed into the main 
      //      nsjsd object. When looping through the file it will
      //      also discover the path to the command file, the path
      //      to the log file, and of course the hosts and aliases.
      //      It will fill two vectors, one with a host list and another
      //      with an alias list.
      //      Unrecognized commands are ignored. Lines that start with # are comments.
      private void extractHosts(){

          FileReader frConfig;      
          BufferedReader brConfig;

          String strHost;
          String strHostTemp;
          String strAlias;
          StringBuffer stbHost;

          int intPortLocal;
          int intPort;

          host hsNew;
          alias alNew;

          try
          {
              //Open the file      
              frConfig = new FileReader(this.fileConfig);
              brConfig = new BufferedReader(frConfig);

              try
              {
                  //Read the first line and continue to read
                  //until the file has been completely parsed
                  strHost = brConfig.readLine();
                  while(strHost != null)
                  {
                      //Don't process any lines unless they're larger than 3 characters
                      if(strHost.length() > 3)
                      {
                          //If this is a host record then extract the host and port information
                          if(strHost.substring(0, 4).equals(new String("host")) == true)
                          {
                              //Remove the word host
                              strHost = new String(strHost.substring(5,strHost.length()));
                              intPortLocal = strHost.indexOf(":");

                              //Extract the host name and the port
                              stbHost = new StringBuffer(strHost.substring(0, intPortLocal));
                              strHostTemp = new String(stbHost);
                              stbHost = new StringBuffer(strHost.substring(intPortLocal + 1, strHost.length()));
                              strHost = new String(stbHost);
                              intPort = Integer.valueOf(strHost).intValue();

                              //Print to the command line the information extracted
                              System.out.println(strHostTemp + ":" + intPort);

                              //Create a new host record and save it to the Vector
                              hsNew = new host(strHostTemp, intPort);
                              lstHosts.add(hsNew);
                          }
                          //If this record points to the command file then extract the path and file name
                          else if(strHost.substring(0, 12).equals(new String("command_file")) == true)
                          {

                              //Extract the name and path of the file
                              strHost = new String(strHost.substring(13, strHost.length()));

                              //Create a new file object and check to see if the command file exists
                              this.fileCommand = new File(strHost);
                              if(this.fileCommand.exists() == false)
                              {
                                  //writelog("The command file doesn't exist, you must specify a valid file");
                                  System.out.println("The command file doesn't exist, you must specify a valid file");
                                  System.exit(0);
                              }
                          }
                          //If this record points to the log file extract the path and name
                          else if(strHost.substring(0, 8).equals(new String("log_file")) == true)
                          {

                              //Extract the name and path of the file
                              strHost = new String(strHost.substring(9, strHost.length()));

                              //Create the new File Object
                              this.fileLog = new File(strHost);

                              //Validate the log file (some preprocessing if the file exists)
                              validatelogfile();
                          }
                          //If this record points to an alias then extract the alias information
                          else if(strHost.substring(0, 5).equals(new String("alias")) == true)
                          {
                              //Remove the word alias
                              strHost = new String(strHost.substring(6,strHost.length()));
                              intPortLocal = strHost.indexOf(":");

                              //Extract the alias name and the host name
                              stbHost = new StringBuffer(strHost.substring(0, intPortLocal));
                              strAlias = new String(stbHost);
                              stbHost = new StringBuffer(strHost.substring(intPortLocal + 1, strHost.length()));
                              strHost = new String(stbHost);

                              //Print to the command line the information extracted
                              System.out.println(strAlias + "-->" + strHost);

                              //Create a new alias record and save it to the Vector
                              alNew = new alias(strHost, strAlias);
                              lstAlias.add(alNew);
                          }
                      }
                      strHost = brConfig.readLine();
                  }

                  frConfig.close();
              }
              catch(IOException ioe)
              {
                  this.fail(ioe, "Reading Exception");
              }
          }
          catch(FileNotFoundException fie)
          {
            this.fail(fie, "File Not Found");
          }
      }
  
      //Name: file Check
      //Desc: This function checks to see if the config file the user passed in
      //      exists. If it does then it returns true, else false.
      //Input: strConfigFile --> String, the full path and name of the file
      //Returns: boolean --> true, if the file exists
      private boolean filecheck(String strConfigFile){
          this.fileConfig = new File(strConfigFile);

          if (this.fileConfig.exists() == true)
          { 
            return true;
          }
          else 
          { 
            System.out.println("The configuration file doesn't exist, you must specify a valid file");
            return false;
          }

      }
  
      /** <B>Name:</B> nsjsd <BR>
       * <B>Desc:</B> This is the only constructor for the object. It takes a single parameter
       * which is the complete path to the configuration file. For syntax of the configuration file
       * see the sample file itself.
       * 
       * @param strConfigFile --> String, the complete path to the configuration file
       */
      public nsjsd(String strConfigFile){

          Vector vecPorts = new Vector();

          host hsTemp;
          Server svrTemp;

          Integer intgPort;

          Date dtStart = new Date();

          int intPort = 0;
          int intPortTemp;
          int i;

          //Set the default port
          intPort = DEFAULT_PORT;

          //Validate the configuration file is okay
          if (this.filecheck(strConfigFile) == false)
            {System.exit(0);}

          //Extract the Hosts and Aliases from the configuration file
          this.extractHosts();

          //Write initialization information to the log file
          writelog("----------------------------------------");
          writelog("Starting nsjsd: " + dtStart.toString());
          writelog("Extracting Host List and Other Parameters");
          writelog("Checking Configuration File");

          //Loop through the list of hosts, and for each host extract a list
          //of ports to listen on.
          for(i=0;i<lstHosts.size();i++)
          {
              //Extract the host record from the vector and it's port number
              hsTemp = (host)lstHosts.get(i);
              intgPort = new Integer(hsTemp.port());

              //If the port vector is empty just add the port
              if(vecPorts.size() == 0)
              {vecPorts.add(intgPort);}
              //If the port vector doesn't alreay contain the port, add it
              else if(vecPorts.contains(intgPort) == false)
              {vecPorts.add(intgPort);}        
          }    

          //Write to the log that we're starting the Servers, one per port
          writelog("Starting one service per port");

          //Loop through the port Vector and start a Server per port. 
          //Then adding each Server to a Vector which holds all the servers.
          for(i=0;i<vecPorts.size();i++)
          {
              //Get the first Port number
              intgPort = (Integer)vecPorts.get(i);
              intPort = intgPort.intValue();

              //Start a new Server with this port number
              svrTemp = new Server(intPort, this);
              svrTemp.start();

              //Add it to the Server Vector
              lstServers.add(svrTemp);
          }

      }  
  
      /** This routine is run when the object is instantiated from the command line. 
       * There is a single mandatory command line parameter, which is the complete path to the configuration file.
       *
       * @param args String --> the complete path to the configuration file.
       */  
      public static void main(String[] args){

          String strConfigFile = new String();

          //Validate that there is at least a single command line arguement
          if (args.length == 1)
          {
            try 
              {strConfigFile = new String(args[0]);}
            catch (Exception e) 
              {System.out.println(e.getMessage());}
          }
          else
          {   //Print to the command line the correct syntax and exit
              printCommandLine();
              System.exit(0);
          }

          //Instantiate a new nsjsd object
          new nsjsd(strConfigFile);
      }
}



